# -*- coding: utf-8 -*-
from django.db.models import Field
from django.conf import settings

from .widgets import AceWidget


class AceField(Field):
    def __init__(self, *args, **kwargs):
        mode = kwargs.pop('mode', None)
        theme = kwargs.pop('theme', None)
        wordwrap = kwargs.pop('wordwrap', False)
        width = kwargs.pop('width', u"500px")
        height = kwargs.pop('height', u"300px")
        showprintmargin = kwargs.pop('showprintmargin', True)
        self.widget = AceWidget(
            mode=mode,
            theme=theme,
            wordwrap=wordwrap,
            width=width,
            height=height,
            showprintmargin=showprintmargin
        )
        super(AceField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "TextField"

    def formfield(self, **kwargs):
        defaults = {'widget': self.widget}
        defaults.update(kwargs)
        return super(AceField, self).formfield(**defaults)


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.conf import settings
from django import forms
from django.forms.utils import flatatt
from django.utils.safestring import mark_safe


class AceWidget(forms.Textarea):
	def __init__(self, mode=None, theme=None, wordwrap=False, width="500px", height="300px", showprintmargin=True,
	             *args, **kwargs):
		self.mode = mode
		self.theme = theme
		self.wordwrap = wordwrap
		self.width = width
		self.height = height
		self.showprintmargin = showprintmargin
		super(AceWidget, self).__init__(*args, **kwargs)
	
	@property
	def media(self):
		js = [
			"djace_editor/ace/ace.js",
			"djace_editor/widget.js",
		]
		if self.mode:
			js.append("djace_editor/ace/mode-%s.js" % self.mode)
		if self.theme:
			js.append("djace_editor/ace/theme-%s.js" % self.theme)
		css = {
			"screen": ["djace_editor/widget.css"],
		}
		return forms.Media(js=js, css=css)
	
	def render(self, name, value, attrs=None, renderer=None):
		attrs = attrs or {}
		
		ace_attrs = {
			"class": "django-ace-widget loading",
			"style": "width:%s; height:%s" % (self.width, self.height)
		}
		toolbar_modes = getattr(settings, 'DJACE_TOLBAR_MODES', {})
		if self.mode:
			ace_attrs["data-mode"] = self.mode
			if self.mode not in toolbar_modes:
				toolbar_modes[self.mode] = self.mode
		if self.theme:
			ace_attrs["data-theme"] = self.theme
		if self.wordwrap:
			ace_attrs["data-wordwrap"] = "true"
		ace_attrs["data-showprintmargin"] = "true" if self.showprintmargin else "false"
		
		textarea = super(AceWidget, self).render(name, value, attrs, renderer)
		
		html = '<div%s><div></div></div>%s' % (flatatt(ace_attrs), textarea)
		toolbar_mode_options = ''
		for k, v in toolbar_modes.items():
			toolbar_mode_options += '<option value="%s">%s</option>' % (k, v)
		
		# add toolbar
		html = '<div class="django-ace-editor">' \
		       '<div style="width: %s" class="django-ace-toolbar">' \
				'<select class="django-ace-mode_select">%s</select>' \
		       '<a href="./" class="django-ace-max_min"></a>' \
		       '</div>%s</div>' % (
			self.width, toolbar_mode_options, html)
		
		return mark_safe(html)



# -*- coding: utf-8 -*-
from setuptools import setup

setup(
    name='djace_editor',
    version='1.0.0',
    description='djace_editor provides integration for ajax.org ACE with Django',
    long_description=open('README.rst').read(),

    author='Alex Vakhitov',
    author_email='alex.vakhitov@gmail.com',
    license="Simplified BSD",
    url='https://github.com/smidth/django-ace-editor',

    packages=['djace_editor'],
    include_package_data=True,
    install_requires=['django>=1.9', 'setuptools'],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
)
